@@ -5,6 +5,7 @@ from typing import List, Union, Any, Optional
 from redis.client import list_or_args
 from redis.exceptions import DataError
 from .conventions import RedizConventions, REDIZ_CONVENTIONS_ARGS, KeyList, NameList, ValueList
+from rediz.utilities import get_json_safe, has_nan, shorten, stem
 
 # REDIZ
 # -----
@@ -47,6 +48,27 @@ class Rediz(RedizConventions):
     def _size(self, name, with_report=False):
         return self._size_implementation(name=name, with_report=with_report, with_private=True)
 
+    def get_donation_password(self):
+        return muid.shash(self._obscurity)
+
+    def donate(self,write_key, password, verbose=True):
+        official_password = self.get_donation_password()
+        if password==official_password:
+            len = muid.difficulty(write_key)
+            if len>=12:
+                self.client.lpush(self.donation_name(len=len),write_key=write_key)
+                return {"operation":"donation","success":True,"message":"Thanks, you can view it at","url":self.base_url+"/donations/"+str(len)} if verbose else 1
+            else:
+                return {"operation":"donation","success":False,"message":"Invalid write key or not long enough"} if verbose else 0
+        else:
+            return {"operation":"donation","success":False,"message":"Invalid password","hint":"Ends with "+str(official_password[-4:])} if verbose else 0
+
+    def get_donations(self, len=None):
+        if len is None:
+            return [ muid.animal(write_key)  for difficulty in range(17,11,-1)  for write_key in self.get_donations(len=difficulty) ]
+        else:
+            return self.client.lrange(self.donation_name(len=len),start=0,end=-1)
+
     def get(self, name, as_json=False, **kwargs ):
         """ Unified getter expecting prefixed name - used by web application """
         parts = name.split(self.SEP)
@@ -74,7 +96,8 @@ class Rediz(RedizConventions):
         return self._get_predictions_implementation(name=name, delay=delay, delays=delays)
 
     def get_cdf(self, name, delay=None, values=None):
-        values = values or self.percentile_abscissa()
+        lags   = self.get_lagged_values(name=name,count=10)
+        values = values or sorted(list(set( self.percentile_abscissa() + (lags or []) )))
         delay  = delay or self.DELAYS[0]
         return self._get_cdf_implementation(name=name, delay=delay, values=values )
 
@@ -84,19 +107,19 @@ class Rediz(RedizConventions):
     def get_delayed(self, name, delay=None, delays=None, to_float=True):
         return self._get_delayed_implementation( name=name, delay=delay, delays=delays, to_float=to_float)
 
-    def get_lagged(self, name, start=0, end=None, count=None, to_float=True ):
+    def get_lagged(self, name, start=0, end=None, count=1000, to_float=True ):
         return self._get_lagged_implementation(name, start=start, end=end, count=count, with_values=True, with_times=True, to_float=to_float)
 
-    def get_lagged_values(self, name, start=0, end=None, count=None, to_float=True):
+    def get_lagged_values(self, name, start=0, end=None, count=1000, to_float=True):
         return self._get_lagged_implementation(name, start=start, end=end, count=count, with_values=True, with_times=False, to_float=to_float)
 
-    def get_lagged_times(self, name, start=0, end=None, count=None, to_float=True):
+    def get_lagged_times(self, name, start=0, end=None, count=1000, to_float=True):
         return self._get_lagged_implementation(name, start=start, end=end, count=count, with_values=False, with_times=True, to_float=to_float)
 
     def get_leaderboard(self, name=None, delay=None, count=50):
         return self._get_leaderboard_implementation(name=name, delay=delay, count=count)
 
-    def get_history(self, name, max='+', min='-', count=None, populate=True, drop_expired=True ):
+    def get_history(self, name, max='+', min='-', count=100, populate=True, drop_expired=True ):
         return self._get_history_implementation( name=name, max=max, min=min, count=count, populate=populate, drop_expired=drop_expired )
 
     def get_subscriptions(self, name ):
@@ -108,6 +131,9 @@ class Rediz(RedizConventions):
     def get_errors(self, write_key, start=0, end=-1):
         return self.client.lrange(name=self.errors_name(write_key=write_key), start=start, end=end)
 
+    def get_warnings(self, write_key, start=0, end=-1):
+        return self.client.lrange(name=self.warnings_name(write_key=write_key), start=start, end=end)
+
     def delete_errors(self, write_key):
         return self.client.delete(self.errors_name(write_key=write_key))
 
@@ -124,18 +150,18 @@ class Rediz(RedizConventions):
         return self.client.hgetall(name=self.performance_name(write_key=write_key))
 
     def get_budget(self, name):
-        return self.client.hget(name=self.BUDGETS,key=name)
+        return self.client.hget(name=self.BUDGET, key=name)
 
     def get_budgets(self):
-        budgets =  self.client.hgetall(name=self.BUDGETS)
-        budgets.sort(key=lambda t: len(t[1]),reverse=True)
-        return budgets
+        budgets =  list(self.client.hgetall(name=self.BUDGET).items())
+        budgets.sort( key=lambda t: t[1],reverse=True)
+        return dict(budgets)
 
     def get_sponsors(self):
         ownership = self.client.hgetall(self._OWNERSHIP)
         obscured = [(name, muid.animal(key)) for name, key in ownership.items()]
         obscured.sort(key=lambda t: len(t[1]))
-        return obscured
+        return dict(obscured)
 
     def delete_performance(self, write_key):
         return self.client.delete(self.performance_name(write_key=write_key))
@@ -147,13 +173,16 @@ class Rediz(RedizConventions):
     def get_backlinks(self, name ):
         return self._get_backlinks_implementation(name=name )
 
-    def get_transactions(self, max='+', min='-', count=None, write_key=None, name=None, delay=None):
+    def get_transactions(self, max='+', min='-', count=50, write_key=None, name=None, delay=None):
         return self._get_transactions_implementation(max=max,min=min,count=count, write_key=write_key, name=name, delay=delay)
 
     def get_summary(self,name):
         assert self._root_name(name)==name
         return self._get_summary_implementation(name)
 
+    def get_home(self,write_key):
+        return self._get_home_implementation(write_key=write_key)
+
     # --------------------------------------------------------------------------
     #            Permissioned get
     # --------------------------------------------------------------------------
@@ -1098,7 +1127,7 @@ class Rediz(RedizConventions):
                 retrieve_pipe.smembers(self._sample_owners_name(name=name, delay=delay))    # List of owners
                 for window_ndx, window in enumerate(self._WINDOWS):
                     scenarios_lookup[name][delay_ndx][window_ndx] = len(retrieve_pipe)
-                    retrieve_pipe.zrangebyscore(name=samples_name, min=value - window, max=value + window, withscores=False)
+                    retrieve_pipe.zrangebyscore(name=samples_name, min=value - window, max=value + window, withscores=False, start=0, num=15)
 
         retrieved = retrieve_pipe.execute()
 
@@ -1125,6 +1154,7 @@ class Rediz(RedizConventions):
 
         # Rewards
         pipe = self.client.pipeline()
+        pipe.hmset(name=self.BUDGET, mapping=dict(zip(names, budgets)))
         for name, budget, write_key in zip(names, budgets, write_keys):
             pools = [retrieved[pools_lookup[name][delay_ndx]] for delay_ndx in range(num_delay)]
             if any(pools):
@@ -1241,7 +1271,7 @@ class Rediz(RedizConventions):
             retrieve_pipe.smembers( self._sample_owners_name(name=name, delay=delay) )          # List of owners
             for window_ndx, window in enumerate(self._WINDOWS):
                 scenarios_lookup[delay_ndx][window_ndx] = len(retrieve_pipe)                    # Robust to insertion of new instructions in the pipeline
-                retrieve_pipe.zrangebyscore( name=samples_name, min=value-window,  max=value+window,  withscores=False, start=0, num=50)
+                retrieve_pipe.zrangebyscore( name=samples_name, min=value-window,  max=value+window,  withscores=False, start=0, num=15)
 
         # Execute pipeline and re-arrange results
         K = 2 + len(self._WINDOWS)
@@ -1304,7 +1334,7 @@ class Rediz(RedizConventions):
     #            Implementation  (getters)
     # --------------------------------------------------------------------------
 
-    def _get_lagged_implementation(self, name, with_times, with_values, to_float, start=0, end=None, count=100 ):
+    def _get_lagged_implementation(self, name, with_times, with_values, to_float, start=0, end=None, count=1000 ):
         count = count or self.LAGGED_LEN
         end = end or start + count
         get_pipe = self.client.pipeline()
@@ -1392,8 +1422,6 @@ class Rediz(RedizConventions):
                 data = self.get_summary(name=parts[-1])
             elif ps == self.SUBSCRIPTIONS:
                 data = self.get_subscriptions(name=parts[-1])
-            elif ps == self.PERFORMANCE:
-                data = self.get_performance(name=parts[-1])
             elif ps == self.SUBSCRIBERS:
                 data = self.get_subscribers(name=parts[-1])
             elif ps == self.LAGGED_VALUES:
@@ -1405,13 +1433,19 @@ class Rediz(RedizConventions):
             elif ps == self.LAGGED_TIMES:
                 data = self.get_lagged_times(name=parts[-1])
             elif ps == self.ERRORS:
-                data = self.get_errors(write_key=parts[-1])
+                data = self.get_errors(write_key=stem(parts[-1]))
+            elif ps == self.PERFORMANCE:
+                data = self.get_performance(write_key=stem(parts[-1]))
             elif ps == self.HISTORY:
                 data = self.get_history(name=parts[-1])
             elif ps == self.BALANCE:
-                data = self.get_performance(write_key=parts[-1])
+                data = self.get_balance(write_key=stem(parts[-1]))
+            elif ps == self.BUDGET:
+                data = self.get_budget(name=parts[-1])
             elif ps == self.TRANSACTIONS:
-                data = self.get_transactions(write_key=parts[-1])
+                data = self.get_transactions(write_key=stem(parts[-1]))
+            elif ps == self.LEADERBOARD:
+                data = self.get_leaderboard(name=parts[-1])
             else:
                 data = None
         elif len(parts) == 3:
@@ -1430,6 +1464,8 @@ class Rediz(RedizConventions):
                 data = self.get_transactions(write_key=parts[1], name=parts[2])
             elif ps == self.PERFORMANCE:
                 data = self.get_performance(name=parts[-1],delay=int(parts[1]))
+            elif ps == self.LEADERBOARD:
+                data = self.get_leaderboard(name=parts[-1],delay=int(parts[1]))
             else:
                 data = None
         else:
@@ -1539,40 +1575,11 @@ class Rediz(RedizConventions):
         return data[0] if singular else data
 
     def _get_summary_implementation(self,name):
-
-        def get_json_safe(thing):
-            data = self.get(thing)
-            if has_nan(data):
-                return None
-            else:
-                try:
-                    json.dumps(data)
-                    return shorten(data)
-                except:
-                    return None
-
-        def has_nan(obj):
-            if isinstance(obj,list):
-                return any( map( has_nan, obj ) )
-            elif isinstance(obj,dict):
-                return has_nan(list(obj.values())) or has_nan(list(obj.keys()))
-            else:
-                try:
-                    return np.isnan(obj)
-                except:
-                    return False
-
-        def shorten(obj):
-            if isinstance(obj,list):
-                return obj[:5]
-            elif isinstance(obj,dict):
-                return dict( [ (k,shorten(v)) for k,v in obj.items() ])
-            else:
-                return obj
+        " Stream summary "
         def delay_level(name,delay):
             things = [ self.leaderboard_name(name=name,delay=delay), self.delayed_name(name=name,delay=delay),self.links_name(name=name,delay=delay),
                        self.cdf_name(name=name,delay=delay)]
-            return dict([(thing, get_json_safe(thing) ) for thing in things])
+            return dict([(thing, get_json_safe(thing=thing,getter=self.get) ) for thing in things])
 
         def top_level(name):
             things = [name, self.lagged_values_name(name), self.lagged_times_name(name),
@@ -1587,6 +1594,18 @@ class Rediz(RedizConventions):
             tl['delay_'+str(delay)] = delay_level(name=name,delay=delay)
         return tl
 
+    def _get_home_implementation(self, write_key):
 
+        def top_level(write_key):
+            things = {'code':muid.shash(write_key),'animal':muid.animal(write_key),
+                       self.balance_name(write_key=write_key):self.get_balance(write_key=write_key),
+                       self.performance_name(write_key=write_key):self.get_performance(write_key=write_key),
+                       self.confirms_name(write_key=write_key):self.get_confirms(write_key=write_key),
+                       self.errors_name(write_key=write_key):self.get_errors(write_key=write_key),
+                       self.warnings_name(write_key=write_key):self.get_warnings(write_key=write_key),
+                       self.transactions_name(write_key=write_key):self.get_transactions(write_key=write_key)}
+            return dict( [ ( thing, shorten(value) ) for thing,value in things.items() ])
+
+        return top_level(write_key=write_key)
 
 
