@@ -51,7 +51,8 @@ class RedizConventions(MicroConventions):
         self.BALANCE = "balance" + self.SEP
         self.PERFORMANCE = "performance" + self.SEP
         self.LEADERBOARD = "leaderboard" + self.SEP
-        self.BUDGETS = "budgets" + self.SEP
+        self.BUDGET = "budget" + self.SEP
+        self.VOLUMES = "volumes" + self.SEP
         self.SUMMARY = "summary" + self.SEP
 
         # Logging
@@ -67,8 +68,8 @@ class RedizConventions(MicroConventions):
         self.CONFIRMS_LIMIT = int(error_limit or 1000)  # Number of error messages to keep per write key
 
         # User transparent temporal and other config
-        self.MIN_LEN = int(self.min_len)                     # FIXME: Get rid of MIN_LEN
-        self.MIN_BALANCE = int(self.min_balance)             # FIXME: Get rid of MIN_BALANCE
+        self.MIN_LEN = int(self.min_len)                     # FIXME: Get rid of MIN_LEN and use min_len instead
+        self.MIN_BALANCE = int(self.min_balance)             # FIXME: Get rid of MIN_BALANCE and use min_balance instead
         self.NUM_PREDICTIONS = int(self.num_predictions)     # Number of scenerios in a prediction batch
         self.DELAYS = delays or [1, 5]
         self.CONFIRMS_MAX = 5  # Maximum number of confirmations when using mset()
@@ -87,6 +88,8 @@ class RedizConventions(MicroConventions):
         self._OWNERS = "owners" + self.SEP  # Prefix to a redundant listing of contemporaneous prediction owners by horizon. Must be private as this contains write_keys
         self._SAMPLES = self._obscurity + self.SAMPLES  # Prefix to delayed predictions by horizon. Contain write_keys !
         self._PROMISED = "promised" + self.SEP  # Prefixes temporary values referenced by the promise queue
+        self._DONATIONS = "donations" + self.SEP  # Write key donations
+        self._DONATION_PASSWORD = muid.shash(self._obscurity)  # Write key donation password
 
         # Other implementation config
         self._DELAY_GRACE = int(delay_grace or 5)  # Seconds beyond the schedule time when promise data expires
@@ -168,6 +171,9 @@ class RedizConventions(MicroConventions):
     def percentile_name(self, name, delay):
         return self.zcurve_name(names=[name],delay=delay)
 
+    def donation_name(self,len):
+        return self._DONATIONS + str(len)
+
     def identity(self, name):
         return name
 
@@ -197,6 +203,9 @@ class RedizConventions(MicroConventions):
     def performance_name(self, write_key):
         return self.PERFORMANCE + write_key + '.json'
 
+    def balance_name(self, write_key):
+        return self.BALANCE + write_key + '.json'
+
     def performance_key(self, name, delay):
         name_stem = os.path.splitext(name)[0]
         return name_stem + self.SEP + str(delay)
